########################################################################################
#                                                                                      #
#   Author: Bertrand Neron,                                                            #
#   Organization:'Biological Software and Databases' Group, Institut Pasteur, Paris.   #  
#   Distributed under GPLv2 Licence. Please refer to the COPYING.LIB document.        #
#                                                                                      #
########################################################################################

"""
The main Class of Mobyle
is also an interface the clients
 - CGI
 
"""


import sys,os
import logging 
import types

try:
    if ( os.path.join( os.environ[ 'MOBYLEHOME' ] , 'Src' ) ) not in sys.path:
        sys.path.append(  os.path.join( os.environ[ 'MOBYLEHOME' ] , 'Src' )  )
except KeyError:
    print >> sys.stderr , "the env vars MOBYLEHOME must be defined "

from Mobyle import MobyleLogger
MobyleLogger.MLogger()


import Mobyle.Net
from Mobyle.MobyleError import *
import Mobyle.Parser
import Mobyle.Job
import Mobyle.Evaluation
import Mobyle.JobState
import Mobyle.Utils


_extra_epydoc_fields__ = [('call', 'Called by','Called by')]

     

class MobyleJob:

    def __init__(self, progUrl= None , email = None , service = None, params = None, evaluator = None , synchron = False , ID = None , session = None , cfg = None ):
        """
        We could instanciate a MobyleJob whith a progName or directly with a Service instance , with params or a Evaluation instance.
        if we instanciate a MobyleJob with a progName mobyleJob will instanciate a service corresponding to this name .
        you could provide the user parameters in a dictionary params which will used to fill an Evalution instance or provide directly the Evaluation instance.
        
        @param progUrl: the url of a xml file
        @type progUrl: String
        @param email: the email where the results will be sent
        @type email: string or L{Mobyle.Net}
        @param service: if you want to run a service which is already instanciate
        @type service: a L{Service} instance
        @param evaluator: an Evaluation instance
        @type evaluator: L{Evaluation} 
        @param synchron: if want to run mobyle in synchron or asynchron mode
        @type synchron: Boolean
        @param ID: the url of job, if an id is specified 
        @type ID: string
        @param cfg: a config object
        @type cfg: L{ConfigManager.Config} instance
        @raise L{MobyleError}: if the id is not a valid url a MobyleError is raised
        @todo: estce que passer un evaluateur a MobyleJob a toujours un sens avec la nouvelle architecture (creation de fichiers ... l'evaluateur a un nom de fichier comme valeur comment recuperer le contenu)
        """
        self.acc_log = logging.getLogger('mobyle.access')
        self.m_log = logging.getLogger('mobyle.mobylejob')

        if cfg is None:
            import Mobyle.ConfigManager
            self.cfg = Mobyle.ConfigManager.Config()
        else:
            self.cfg = cfg
            
        if ID:
            self.jobState = Mobyle.JobState.JobState( ID )
            self._job = None
            self._hasRun = True
        else:
            self.build_log = logging.getLogger('mobyle.builder')
            
            if progUrl:
                jobName = os.path.basename( progUrl )[:-4]
            elif service:
                jobName = service.getName()
            else:
                raise MobyleError, "you must provide either a service or a service name"

            ## check the service availablilty
            if self.cfg.isDisabled( jobName  ):
                if self.cfg.isDisabled():

                    raise MobyleError, "Sorry, the server is not available for now."
                else:
                    raise MobyleError, "Sorry, the program " + jobName +" is  not available for now."

            try:
                remoteHost = os.environ[ 'REMOTE_HOST' ]
            except KeyError:
                remoteHost = None

            try:
                ip = os.environ[ 'REMOTE_ADDR' ]
            except KeyError :
                ip = 'local'

            self._remote =( ip , remoteHost )
            
            if  self._remote[1]:
                remoteLog = self._remote[1]  #the remote host    
            else:
                 remoteLog  = self._remote[0] #the remote addr
                 
            if not email and session :
                email = session.getEmail()

            try:
                Mobyle.Net.checkHost()
            except UserValueError ,err:
                               
                msg = "%s %s %s %s %s" %( jobName,
                                       email ,
                                       remoteLog ,
                                       "FORBIDDEN ",
                                       err
                                      )
                
                self.m_log.info( msg )
                
                raise UserValueError( msg = "you are not allowed to run a job on this server for now " )

            if email:
                if  isinstance( email , types.StringTypes ):
                    self.email = Mobyle.Net.Email( email , self.cfg )
                else:
                    self.email = email

                chk = self.email.check()
                if not chk:
                    msg = self.email.getMessage()
                    userMsg = "you are not allowed to run on this server for now "
    
                    msg = "%s %s %s %s" %( jobName ,
                                           self.email.To ,
                                           remoteLog ,
                                           "FORBIDDEN "+ msg
                                           )
    
                    self.m_log.info( msg )
                     
                    raise UserValueError( msg = userMsg )          
            
            else:
                if self.cfg.opt_email( jobName = jobName ):
                    self.email = None
                else:
                    raise MobyleError , "the email is required"
 
            ## building the MobyleJob

            self.progUrl = progUrl
            self._service = service
            self.params = params # a garder??
            self._evaluator = evaluator
            self.synchron = synchron
            self.session = session
            self._job = None             #an instance of Mobyle.Job
            self.jobState = None       #an instance of Mobyle.JobState.JobState
            self._adm = None
            self._hasRun = False
        
            
            if params is None:
                if service is None:
                    if progUrl is None:
                        ### TODO il n'y a pas encore de job de cree donc logger cette erreur ?
                        raise MobyleError, "you must provide either a service or a service url"
                    else:
                        if evaluator is None or not evaluator.isFill():
                            #self._service = self._makeService( progUrl )
                            try:
                                self._service = Mobyle.Utils.makeService( progUrl )
                            except MobyleError , err :
                                self._logError( admMsg = "MobyleJob._init_ (L195) : " + str( err ) ,
                                                userMsg = "Mobyle internal server error"
                                                )
                                raise MobyleError , str( err )

                            self._service = Mobyle.Utils.makeService( progUrl )
                            self._evaluator = self._service.getEvaluator()
                            
                        else:
                            if evaluator.isFill():
                                try:
                                    self._service = Mobyle.Utils.makeService( progUrl )
                                except MobyleError ,msg :
                                    self._logError( admMsg = "MobyleJob._init_ (L203) : " + str( err ) ,
                                                    userMsg = "Mobyle internal server error"
                                                    )
                                    raise MobyleError , str( err )

                                self._service.setEvaluator( evaluator )
                               

                else: #service is not None
                    if self._evaluator is None or not self._evaluator.isFill():
                        self._evaluator = self._service.getEvaluator()
                           
                    else:
                        self._service.setEvaluator( evaluator )
                                                    
                self._debug = self.cfg.debug( self._service.getName() )
                self._job = Mobyle.Job.Job( self._service ,
                                            self.cfg ,
                                            email = self.email ,
                                            synchron= self.synchron ,
                                            session = self.session
                                            )
                                

            else: #params is not None
                if service is None:
                    if progUrl is None:
                        raise MobyleError,"you must provide either a service a service url"
                    else:
                        self._service = Mobyle.Utils.makeService( progUrl )
                        self._evaluator = self._service.getEvaluator()
                         
                else: #service is not None
                    self._evaluator = Mobyle.Evaluation.Evaluation()
                    self._service.setEvaluator( self._evaluator )

                self._debug = self.cfg.debug( self._service.getName() )
                self._job = Mobyle.Job.Job( self._service ,
                                            email = self.email ,
                                            synchron= self.synchron ,
                                            session = self.session
                                            )
               
                #self._fillEvaluator( params )

                
            ############ in all cases except ID ###################

            jobname =  self._service.getName()
            
                
            if self._debug > 1:
                self.build_log.debug( "MobyleJob : params= " + str( params ) )


            self.jobState = self._job.jobState
            self._adm = Mobyle.Utils.Admin( self._job.getDir() )
                    
                




    ###############################################################
    #
    #                
    #
    ###############################################################

    def setSession(self , session ):
        self.session = session
        
        if self._job is not None:
            self._job.setSession( session )
            
        
    def setValue( self , paramName , value ):
        """
        set the value for one parameter. the value is cast in the rigth type. if the value cannot be cast a UserValueError is raised
        @param paramName: the name of the parameter
        @type paramName: string
        @param value : the value of the parameter
        @type value:
           - for simple type: any
           - for the infile :a tuple ( destFileName , content , src , srcFileName )
             - the name of the file
             - the data the content of the file , string or None 
             - the src where the file could be retreive, MobyleJob , Job or Sesion instance or None
        data is specified when datas are new on the server.
        src is specified when the data already exist on the server.
        you can't specify data and src in the same time
        @raise: MobyleError if the paramName doesn't match with any parameter name
        """

        if self._hasRun:
            msg = "MobyleJob.setValue this job has already ran"
            self._logError( admMsg = "MobyleJob.setValue : " + msg ,
                            logMsg = msg ,
                            userMsg = msg
                            )

            raise MobyleError , msg

        
        
        if self._debug > 1:
            self.build_log.debug( "\n--------------------- MobyleJob set user value for " + paramName + "--------------------" )        
        if paramName in  self._service.getAllParameterNameByArgpos():

            if value == '' :
                value = None

            if self._service.isInfile( paramName ):

                if len( value ) == 4 : ###MobyleJob is called from a cgi
                    destFileName , data  , src , srcFileName = value

                    if not destFileName and not data and not src:
                        data  = None
                        src = None
                       
                    elif destFileName and not ( data or src ):
                        msg = "parameter %s: is empty. please check your data" % paramName
                        self._logError( admMsg = "Mobyle.setValue: "+ msg ,
                                        logMsg = msg ,
                                        userMsg = msg
                                        )
                        param = self._service.getParameter( paramName )
                        raise UserValueError( parameter =  param , msg = msg )

                    elif data and src:
                        msg = "parameter %s: you cannot specify data and source at the same time" % paramName
                        self._logError( admMsg = "Mobyle.setValue: "+ msg ,
                                        logMsg = msg ,
                                        userMsg = msg
                                        )
                        param = self._service.getParameter( paramName )
                        raise UserValueError( parameter =  param , msg = msg )
                        

                    elif not srcFileName and not data and src:
                        msg = "parameter %s: if you specify a src, you must specify a src file name" % paramName
                        self._logError( admMsg = "Mobyle.setValue: "+ msg ,
                                        logMsg = msg ,
                                        userMsg = msg
                                        )
                        param = self._service.getParameter( paramName )
                        raise UserValueError( parameter =  param , msg = msg )
                       
                    elif not srcFileName and data and not src:
                        fileName = paramName + ".data"
                

                    try:
                        self._service.setValue( paramName , (  data , self._job , destFileName , src ,srcFileName ) )
                        if self._debug > 1:
                            self.build_log.debug("self._service.setValue( %s , %s )" %( paramName ,
                                                                               ( destFileName ,
                                                                                 data ,
                                                                                 self._job ,
                                                                                 src
                                                                                 )
                                                                               )
                                        )
                    except UserValueError ,err:
                        msg =  str( err )
                        self._logError( admMsg = "MobyleJob.setValue : " + msg ,
                                        logMsg = msg ,
                                        userMsg = msg
                                        )
                        parameter = self._service.getParameter( paramName )
                        raise UserValueError( parameter = parameter , msg = msg )
                    
                    # I put sytematicaly the value in index.xml even if its the
                    # same as the previous value beacuse the value is a filename 
                    # and 2 files with the same name could have different contents
                    value = self._service.getValue( paramName )
                    lang = self.cfg.lang()
                    prompt = self._service.getPrompt( paramName , lang = lang )
                    #self.jobState.addArg( paramName , value , prompt = prompt )
                    #self.jobState.commit()
                    
                else:
                    msg = "Problem with the infile : " + paramName
                    self._logError( admMsg = "Mobyle.setValue: "+msg ,
                                    logMsg = msg ,
                                    userMsg = "Internal Mobyle Server Error"
                                    )
                    parameter = self._service.getParameter( paramName )
                    raise UserValueError( parameter = parameter , msg = msg )
                    
            else : #the parameter is not an infile
             
                oldValue = self._service.getValue( paramName )
             
                try:
                    self._service.setValue( paramName , value )
                    if self._debug > 1:
                        self.build_log.debug( "self._service.setValue( %s , %s )" %( paramName ,
                                                                            value )
                                     )

                except UserValueError , err:
                        msg =  str( err )
                        self._logError( admMsg = "MobyleJob.setValue : " + msg ,
                                        logMsg = msg ,
                                        userMsg = msg
                                        )
                        raise UserValueError , err
                    
                newValue = self._service.getValue( paramName )
                
                if newValue != oldValue :
                
                    rawVdef =  self._service.getVdef( paramName )
                    if rawVdef is None:
                        vdef = None
                    else:
                        vdef = self._service.convert( paramName , rawVdef )
                
                    if newValue == vdef :
                        if oldValue is not None :
                            self.jobState.delInputData( paramName )
                            self.jobState.commit()
                    else:
                        lang = self.cfg.lang()
                        prompt = self._service.getPrompt( paramName , lang = lang )
                        self.jobState.setInputDataValue( paramName , ( prompt , lang ) , self._service.getType(paramName) , newValue )
                        self.jobState.commit()
            
        else:
            msg = "the parameter named : %s doesn't exist in %s" %( paramName ,
                                                                    self._service.getName()
                                                                    )

            self._logError( admMsg = "MobyleJob.setValue : " + msg ,
                            logMsg = msg ,
                            userMsg = msg
                            )

            raise UserValueError( msg = msg )



    def _validateParameters(self):
        """
        verifie que le parametre est valide cad si c'est un choice qu'il appartient bien a la liste si c'est une sequence passe dans squizz ...
        @raise UserValueError: throws UserValueError raised by checkUser
        """
        
        if self._debug > 1:
            self.build_log.debug("""\n
              \t################################################
              \t#                                              #
              \t#              validation begining             #
              \t#                                              #
              \t################################################
              \n""")
                
        for paramName in self._service.getAllParameterNameByArgpos():
            ###################################################
            #                                                 #
            #         check if parameter have a value         #
            #                                                 #
            ###################################################
            
            value = self._service.getValue(paramName)
            if self._debug > 1:
                self.build_log.debug("------------- " + paramName + " -------------")
                self.build_log.debug("value= %s : type= %s"%( value , type(value) ) )
                self.build_log.debug("service.precondHas_proglang( %s , 'python' ) = %s " %(
                    paramName ,
                    self._service.precondHas_proglang( paramName , 'python' )
                    )
                            )
                    
                
            if self._service.precondHas_proglang( paramName , 'python' ):
                preconds = self._service.getPreconds( paramName , proglang='python' )
                allPrecondTrue = True
               
                for precond in preconds:
                    evaluatedPrecond = self._evaluator.eval( precond )
                    
                    if self._debug > 1:
                        self.build_log.debug("precond= " + precond )
                        self.build_log.debug("eval precond= " + str( evaluatedPrecond ))
                        
                    if not evaluatedPrecond :
                        allPrecondTrue = False
                        break
                
                if not allPrecondTrue :
                    if self._debug > 1:
                        self.build_log.debug("next parameter")
                    continue #next parameter

            else:
                if self._debug > 1:
                    if self._service.precondHas_proglang( paramName, 'perl' ) :
                        self.build_log.debug( "################ WARNING ###################" )
                        self.build_log.debug( "had a precond code in Perl but not in Python" )
                        self.build_log.debug( "############################################" )

                
            if value is None :
                if self._debug > 1:
                    self.build_log.debug( " value is None " )
                if self._service.ismandatory( paramName ):
                    msg = "The mandatory Parameter : %s, has no value " % paramName
                    
                    self._logError( admMsg = "MobyleJob._validateParameters : " + msg ,
                                    logMsg = msg ,
                                    userMsg = msg
                                    )

                    raise  UserValueError(
                                          parameter = self._service.getParameter( paramName ) ,
                                          msg = "This parameter is mandatory" )
            
            if self._debug > 1:
                self.build_log.debug( "call service.validate( " + paramName + " )" )
            try:    
                self._service.validate( paramName )
            except UserValueError , err:
                msg = str(err)
                self.build_log.debug( msg )
                self._logError( admMsg = "MobyleJob._validateParameters : " + msg ,
                                logMsg = msg ,
                                userMsg = msg
                                )
                raise UserValueError ,err
            
            except MobyleError , err:
                msg = str(err)
                self.build_log.debug( msg )
                self._logError( admMsg = "MobyleJob._validateParameters : " + msg ,
                                logMsg = msg ,
                                userMsg = "Mobyle Internal Server Error"
                                )                
                raise MobyleError , err
            ####################################################
            #                                                  #
            #  check if the Parameter have a secure paramfile  #
            #                                                  #
            ####################################################

            try:
                if self._debug > 1:
                    self.build_log.debug( "check if the Parameter have a secure paramfile" )

                paramfile = self._service.getParamfile( paramName )
                try:
                    safeParamfile = Mobyle.Utils.safeFileName( paramfile )
                except UserValueError , err :
                    msg = str( err )
                    self._logError( admMsg = "MobyleJob._validateParameters : " + msg ,
                                    logMsg = msg ,
                                    userMsg = msg
                                    )
                    raise UserValueError( parameter= self._service.getParameter( paramName ), msg = msg )
                
                if not safeParamfile or safeParamfile != paramfile :
                    msg = "The Parameter:%s, have an unsecure paramfile value: %s " %(  paramName ,
                                                                                        paramfile
                                                                                        )
                    if self._debug < 2 :
                        self._logError( admMsg = "MobyleJob._validateParameters : " + msg ,
                                        logMsg = None ,
                                        userMsg = "Mobyle Internal Server Error"
                                        )
                        
                        self.m_log.critical( "%s/%s : %s" %( self._service.getName(),
                                                          self._job.getKey() ,
                                                          msg
                                                          )
                                        )
                    else:
                        self._logError( admMsg = "MobyleJob._validateParameters : " + msg ,
                                        logMsg =  msg ,
                                        userMsg = "Mobyle Internal Server Error"
                                        )


                    raise MobyleError , "Mobyle Internal Server Error"
                else:
                    if self._debug > 1:
                        self.build_log.debug( "paramfile = %s ...........OK" % paramfile )
            except UnDefAttrError :
                if self._debug > 1:
                    self.build_log.debug("no paramfile")
                pass
            

    def run(self):
        """
        run a job
        @raise MobyleError: L{MobyleError} if the job dosen't exist. or if session is not activated
        """
        if self._job is not None:
            
            if not self.cfg.isAuthorized( self._service.getName() , self._remote[0] ):
                msg = "Sorry, you are not allowed to run this service."
                admmsg = " %s try to access restricted service"%self._remote[0]
                self._logError( admMsg = admmsg , 
                                userMsg = msg , 
                                logMsg = admmsg,
                                )                
                raise MobyleError, msg
            
            if self.session and not self.session.isActivated():
                
                msg = "session \"%s\" is not activated : run aborted" % self.session.getKey()
                
                self._logError( admMsg = msg , 
                                userMsg = "session is not activated : run aborted" , 
                                logMsg = msg 
                                )
                
                raise MobyleError , "session is not activated"
            
            # for security reason 
            # I remove some unsuful variable form environment
            # to avoid to propagate them in all cluster nodes
            #in Waiting a better momdel 
            
            for envVar in [ 'CONTENT_LENGTH'      ,
                            'CONTENT_TYPE'        ,
                            'GATEWAY_INTERFACE'   ,
                            'HTTP_ACCEPT'         ,
                            'HTTP_ACCEPT_ENCODING',
                            'HTTP_ACCEPT_LANGUAGE',
                            'HTTP_CACHE_CONTROL'  ,
                            'HTTP_ACCEPT_CHARSET' ,
                            'HTTP_COOKIE'         ,
                            'HTTP_CONNECTION'     ,
                            'HTTP_KEEP_ALIVE'     ,
                            'HTTP_PRAGMA'         ,
                            'HTTP_REFERER'        ,
                            'HTTP_USER_AGENT'     ,
                            'HTTP_X_PROTOTYPE_VERSION',
                            'HTTP_X_REQUESTED_WITH',
                            'QUERY_STRING'        ,
                            'SERVER_ADMIN'        ,
                            'REQUEST_METHOD'      ,   
                            'REQUEST_URI'         ,
                            'SCRIPT_FILENAME'     ,
                            'SCRIPT_NAME'         ,
                            #'SERVER_NAME'         ,
                            #'SERVER_PORT'         ,
                            'SERVER_PROTOCOL'     ,
                            'SERVER_SIGNATURE'    ,
                            'SERVER_SOFTWARE'     ,

]:
                try:
                    del( os.environ[ envVar ])
                except KeyError:
                    pass
                    
            self._validateParameters()
                    
            #do the controls specified in the Xml file
            if self._debug > 1:
                self.build_log.debug("""\n
                \t################################################
                \t#                                              #
                \t#           xml controls begining              #
                \t#                                              #
                \t################################################
                \n""")
            
            for paramName in self._service.getAllParameterNameByArgpos():
             
                hasCtrl = self._service.has_ctrl( paramName )
                hasScale = self._service.hasScale( paramName , proglang = 'python' )
             
                if hasCtrl or hasScale:

                    if self._debug > 1:
                        self.build_log.debug("------------- " + paramName + " -------------")
                        self.build_log.debug("service.precondHas_proglang( %s , 'python' ) = %s " %(
                            paramName ,
                            self._service.precondHas_proglang( paramName , 'python' )
                            ))
                        
                    if self._service.precondHas_proglang( paramName , 'python' ):
                        preconds = self._service.getPreconds( paramName , proglang='python' )
                        allPrecondTrue = True
                       
                        for precond in preconds:
                            evaluatedPrecond = self._evaluator.eval( precond )
                            
                            if self._debug > 1:
                                self.build_log.debug("precond= " + precond )
                                self.build_log.debug("eval precond= " + str( evaluatedPrecond ))
                                
                            if not evaluatedPrecond :
                                allPrecondTrue = False
                                break
                        
                        if not allPrecondTrue :
                            if self._debug > 1:
                                self.build_log.debug("next parameter")
                            continue #next parameter
        

                    if hasCtrl:
                        if self._service.getValue( paramName ) is not None:
                            try:
                                self._service.doCtrls( paramName )
                            except ServiceError , err:
                                # when the parameterName doesn't exist 
                                # or an error occured during the python code evaluation ( see parameter.doCrtls() )
                                msg = str( err )
                                self._logError(
                                    admMsg = "MobyleJob.run : " + msg ,
                                    logMsg = msg ,
                                    userMsg = "Internal Mobyle server Error"
                                    )
                                raise MobyleError , msg 
                            except UserValueError , err:
                                msg = str( err )
                                self._logError(
                                    admMsg = "MobyleJob.run : " + msg ,
                                    logMsg = msg ,
                                    userMsg = msg
                                    )
                                raise err
                                
                                
                        else:
                            if self._debug > 1:
                                self.build_log.debug("undefined value: next parameter")
                            continue

                    if hasScale:
                        if self._debug > 1:
                            self.build_log.debug( paramName + " has scale= True" )

                        try:
                            isInScale = self._service.isInScale( paramName , proglang = 'python')

                            if self._debug > 1:
                                self.build_log.debug( "isInScale= " + str( isInScale ) )

                            if isInScale:
                                continue
                            else:
                                smin ,smax , incr = self._service.getScale( paramName , proglang = 'python') 
                                msg = "%s value: %s is not in scale ( %s , %s )" %(
                                    paramName ,
                                    self._service.getValue( paramName ) ,
                                    smin ,
                                    smax )

                                self._logError(
                                    admMsg = "MobyleJob.run : " + msg ,
                                    logMsg = msg ,
                                    userMsg = msg
                                    )
                                parameter = self._service.getParameter( paramName )
                                raise UserValueError( parameter = parameter , msg = msg )
                            
                        except ValueError , err :
                            msg = paramName + " value is None. Thus it is not in scale"
                            self._logError(
                                admMsg = "MobyleJob.run : " + msg ,
                                logMsg = msg ,
                                userMsg = msg
                                )
                            raise MobyleError , msg
                    else:
                        if self._debug > 1:
                            self.build_log.debug("has scale= False" )

                            continue
                        
            if  self._remote[1]:
                remoteLog = self._remote[1]  #the remote host    
            else:
                 remoteLog  = self._remote[0] #the remote addr
                 

            if self.email:
                msg = "%s %s %s %s" %(self._service.getName() ,
                                                 self._job.getKey() ,
                                                 self.email.To ,
                                                 remoteLog
                                                 )
            else:
               msg = "%s %s None %s" %(self._service.getName() ,
                                          self._job.getKey(),
                                          remoteLog
                                          )   
            self.acc_log.info( msg )
                                             
            self._hasRun = True
            cmd = self._job.run()  
            return self._job.getURL()
        
        else:
            if self._hasRun:
                msg = "MobyleJob.run this job has already ran"
            else:
                msg = "MobyleJob.run: can't run the job (%s). It doesn't exist" % self._job.getURL() 

            self._logError( admMsg = "MobyleJob.run : " + msg ,
                            logMsg = msg ,
                            userMsg = msg
                            )

            raise MobyleError , msg 
        


      
    
    ####################################################################
    #
    #                    get job info
    #
    ####################################################################

    def getDir(self):
        """
        @return: the directory absolute path where the job is executed
        @rtype: string
        @call: cgiJob.printResult()
        """
        
        if self._job:
            return self._job.getDir()
        elif self.jobState:
            url = self.jobState.getID()
            return Mobyle.JobState.url2path( url )
        else:
            return None


    def getDate(self):
        """
        @return: the submission date of this job
        @rtype: string
        @call: cgiJob.submit()
        """
        
        if self._job:
            return self._job.getDate()
        else:
            return None     
           

    def getJobid(self):
        """
        @return: url where the job is executed
        @rtype: string
        """
        if self.jobState:
            return self.jobState.getID()
        else:
            return None
        

    def getOutputs(self):
        """
        @return: a dictionnary where the keys are the parameter name and the values a list of files which are produced by this parameter.
        @rtype: dict

        """
        if self.jobState:
            if self._hasRun:
                return self.jobState.getOutputs()
            else:
                msg = "try to get results but the job had not run"
                raise MobyleError , msg
        else:
            return None

    def getOutput(self , paramName ):
        """
        @return: the list of results files names. if there is no files for this parameter return None 
        @rtype: list of tuples ( string filename , long size , string fmt or None )
        """
        if self.jobState:
            if self._hasRun:
                return self.jobState.getOutput( paramName )
            else:
                msg = "try to get result but the job had not run"
                raise MobyleError , msg
        else:
            return None


    def getOutputFile( self, fileName ):
        """
        @param fileName:
        @type fileName: String
        @return: the content of a output file as a string
        @rtype: string
        @raise exception: 
        """
        if self.jobState:
            if self._hasRun:
                return self.jobState.getOutputFile( fileName )
            else:
                msg = "try to get result but the job had not run"
                raise MobyleError , msg
        else:
            return None  
        
        
    def isLocal( self ):
        """
        
        """
        if self.jobState:
                return self.jobState.isLocal()



    def getStatus(self):
        """
        @return: None or The status of the job and the message associated with.
          The status value could be 'building, 'suspended , 'submitted' , running' , 'finish', 'error'
          and a message could be associated.
          If there is no message return an empty string.
        @rtype: (string status, String message)
        """
        if self.jobState:
            return Mobyle.Utils.getStatus( self.jobState.getID() )
            
        else:
            return  ( None , '' ) 




    def isFinished(self):
        """
        @return: True if the job is finished, False Otherwise
        @rtype: boolean
        """
        if self.jobState:
            return self.jobState.isFinished()
        else:
            return None



    def getCommandLine(self):
        """
        @return: the Command line
        @rtype: string
        """
        if self.jobState:
            if self._hasRun:
                return self.jobState.getCommandLine()
            else:
                msg = "try to get the command but the job had not run"
                raise MobyleError , msg
        else:
            return None

    def getDataPrompt(self , paramName ):
        """
        @return: the prompt for an In/Output parameter
        @rtype: string
        """
        if self.jobState:
            if self._hasRun:
                return self.jobState.getPrompt( paramName )
            else:
                msg = "try to get the command but the job had not run"
                raise MobyleError , msg
        else:
            return None
    
    
    def getStdout(self):
        """
        @return: the content of the job stdout as a string
        @rtype: string
        @raise MobyleError: if the job is not finished raise a L{MobyleError}
        """
        if self.jobState:
             if self._hasRun:
                 return self.jobState.getStdout()
             else:
                msg = "try to get the job stdout, but the job had not run"
                raise MobyleError , msg
        else:
            return None
    

    def getStderr(self):
        """
        @return: the content of the job stderr as a string
        @rtype: string
        @raise MobyleError: if the job is not finished raise a L{MobyleError}
        """
        if self.jobState:
            return self.jobState.getStderr()
        else:
            return None

    
    def open(self , fileName ):
        """
        return an file object if the file is local or a file like object if the file is distant
        we could apply the same method on this object: read(), readline(), readlines(), close(). (unlike file the file like object doesn't implement an iterator).
        @param fileName: the name of the file (given by getResults).
        @type fileName: string
        @return: a file or file like object, or None if the job isn't ran.
        """

        if self.jobState:
            return self.jobState.open( fileName )
        else:
            return None



    def getArgs(self):
        """
        read the parameters in the .index.xml file and return a dictionary containig these parameters
        @return: the parameters of a job
        @rtype: dictionnary
        """

        if self.jobState:
            return self.jobState.getArgs()
        else:
            return None

        
    def _logError( self , admMsg = None , userMsg = None , logMsg = None ):
       
        if admMsg and self._adm:
            self._adm.setStatus( 5 , admMsg )
            self._adm.commit()

        if userMsg:
            self.jobState.setStatus( 5 , userMsg )
            self.jobState.commit()

        if  logMsg :
            self.m_log.error( "%s/%s : %s" %( self._service.getName() ,
                                           self._job.getKey() ,
                                           logMsg
                                           )
                         )


           ##########################################################
        

